from .assumption_spec import AssumptionSpec
from .cell_extraction_spec import CellExtractionSpec
from .locator_spec import LocatorSpec
from .validator_spec import ValidatorSpec
from .excel_processor_spec import ExcelProcessorSpec
